@@ -22,7 +22,6 @@ def _create_command_list(
     command: str,
     flags_dict: Mapping[str, Any],
 ) -> Sequence[str]:
-
     prefix = [executable]
     if warn_error:
         prefix += ["--warn-error"]
@@ -63,7 +62,7 @@ def execute_cli(
 ) -> DbtCliOutput:
     """Executes a command on the dbt CLI in a subprocess."""
     try:
-        import dbt  # pylint: disable=unused-import
+        import dbt  # noqa: F401
     except ImportError as e:
         raise check.CheckError(
             "You must have `dbt-core` installed in order to execute dbt CLI commands."
@@ -109,7 +108,6 @@ def execute_cli(
 
         log_level = "info"
         message = line
-
         raw_output.append(line)
 
         if json_log_format:
@@ -123,6 +121,12 @@ def execute_cli(
                 if isinstance(json_line, dict):
                     message = json_line.get("message", json_line.get("msg", message))
                     log_level = json_line.get("levelname", json_line.get("level", "debug"))
+        elif "Done." not in line:
+            # attempt to parse a log level out of the line
+            if "ERROR" in line:
+                log_level = "error"
+            elif "WARN" in line:
+                log_level = "warn"
 
         messages.append(message)
         if capture_logs:
